/*  
 * Function genPalette -
 * 
 * this function generates a generic palette for use in Rippuken. 
 *
 * Parameters:
 *  quantity specifies how many colors are to be in the palette.
 *  dst is a pointer to the starting address the palette is to be written to.
 *
 *  A palette entry consists of a 3 byte triplet of RGB values.
 *  byte 0 is red, byte 1 is green, byte 2 is blue
 *  0x00 is 0 intensity, 0xff is full intensity
 *
 *  In this function, the palette is written to memory byte by byte,
 *  colour by colour, in the order R-G-B.  It is assumed that the required 
 *  memory has already been allocated in the heap by the calling function 
 *  and that the starting address of the allocated memory is given by *dst.
 *  The function also assumes correct quantity (>=1) is given.
 */
void genPalette(char *dst, int quantity)
{
    int rsteps = 0;
    int gsteps = 0;
    int bsteps = 0;
    unsigned char rvalue = 0;
    unsigned char gvalue = 0;
    unsigned char bvalue = 0;
    int loopindex = 0;
    int quotient = 0;
    int remainder = 0;
    int extra = 0;
    int i;

    /* the first colour is always black */
    *dst = rvalue;
    dst += 1;
    *dst = gvalue;
    dst += 1;
    *dst = bvalue;
    dst += 1;

    if ((quantity - 1) <= 7) {
	/* do a palette from R, RG, G, GB, B, BR, RGB ending at whatever
	 * the quantity ends at. */
	for (i = 1; i <= quantity; i++) {
	    switch (i) {
	    case 1:
		rsteps = 0xff;
		break;
	    case 2:
		rsteps = 0;
		gsteps = 0xff;
		break;
	    case 3:
		rsteps = -0xff;
		gsteps = 0;
		break;
	    case 4:
		rsteps = 0;
		bsteps = 0xff;
		break;
	    case 5:
		gsteps = -0xff;
		bsteps = 0;
		break;
	    case 6:
		rsteps = 0xff;
		gsteps = 0;
		break;
	    case 7:
		rsteps = 0;
		gsteps = 0xff;
		break;
	    default:
		break;
	    }
	    rvalue += rsteps;
	    gvalue += gsteps;
	    bvalue += bsteps;
	    *dst = rvalue;
	    dst += 1;
	    *dst = gvalue;
	    dst += 1;
	    *dst = bvalue;
	    dst += 1;
	}
    } else {
	/* work out the colour depths between each colour transition, and
	 * then produce the palette.
	 * if there is remainder, then they are used to add greyscale 
	 * palettes at the end.
	 */
	quotient = (quantity - 1) / 7;
	remainder = (quantity - 1) % 7;
	extra = 0xff % quotient;
	rvalue = extra;
	gvalue = extra;
	bvalue = extra;
	rsteps = 0xff / quotient;
	gsteps = 0xff / quotient;
	bsteps = 0xff / quotient;
	loopindex = quantity - 1 - remainder;
	while (loopindex > 0) {
	    if ((gvalue < 0xff) && (bvalue == 0xff) && (rvalue == 0xff))
		gvalue += gsteps;
	    if ((gvalue == extra) && (bvalue == 0xff) && (rvalue < 0xff))
		rvalue += rsteps;
	    if ((gvalue > extra) && (bvalue == 0xff) && (rvalue == extra))
		gvalue -= gsteps;
	    if ((gvalue == 0xff) && (bvalue < 0xff) && (rvalue == extra))
		bvalue += bsteps;
	    if ((gvalue == 0xff) && (bvalue == extra) && (rvalue > extra))
		rvalue -= rsteps;
	    if ((gvalue < 0xff) && (bvalue == extra) && (rvalue == 0xff))
		gvalue += gsteps;
	    if ((gvalue == extra) && (bvalue == extra) && (rvalue < 0xff))
		rvalue += rsteps;
	    *dst = rvalue;
	    dst += 1;
	    *dst = gvalue;
	    dst += 1;
	    *dst = bvalue;
	    dst += 1;
	    loopindex--;
	}
	if (remainder > 0) {
	    rsteps = 0xff / (remainder + 2);
	    gsteps = rsteps;
	    bsteps = rsteps;
	    rvalue = 0;
	    gvalue = 0;
	    bvalue = 0;
	    for (i = 1; i <= remainder; i++) {
		rvalue += rsteps;
		gvalue += gsteps;
		bvalue += bsteps;
		*dst = rvalue;
		dst += 1;
		*dst = gvalue;
		dst += 1;
		*dst = bvalue;
		dst += 1;
	    }
	}
    }
}
